@echo off
setlocal enabledelayedexpansion

echo Airspy HF+ Flash Utility

set currentDir=%~dp0
cd %currentDir%
set /a attempt=0

echo Looking for a suitable flashable device...

:queryDevice
    for /f "skip=1 tokens=1,2 delims= " %%a in ('wmic path Win32_SerialPort get PNPDeviceID^,DeviceID^') do (
        set DeviceID=%%a
        set PNPDeviceID=%%b
        if "!PNPDeviceID:~4,17!"=="VID_03EB&PID_6124" (
            echo Flashable device found on port !DeviceID!
            goto :flashDevice
        )
    )

    if "%oemInf%" == "" (
      echo Looking for a suitable flashing driver...
      for /f "tokens=1,2 delims=:" %%a in ('pnputil -e ^| find /I "oem"') do (        
          set oemInf=%%b
          call :Trim !oemInf! oemInf
          for /f "skip=1 tokens=1,2,3 delims=-: " %%a in ('find /C /I "arduino-org.inf" %windir%\INF\!oemInf!') do (
              echo This one can do the job: %%b
              hfplus_fw.exe | findstr /c:"CD" 1>nul
              if errorlevel 1 (
                set fwbin=hfplus-firmware-bb.bin
              ) else (
                set fwbin=hfplus-firmware-cd.bin
              )
              echo Saving the calibration...
              hfplus_ppb.exe > Calib
              echo Rebooting the device in flash mode...
              hfplus_reboot.exe > NUL
              timeout 5 > NUL
              goto :queryDevice
          )
      )

      if !attempt! == 0 (
          echo No suitable driver found.
          call :checkIfAdminOrElevated
          if !errorLevel! NEQ 0 goto :end
          call :checkWindowsArchitecture
          call :installDriver
          set /a attempt=1
          goto :queryDevice
      ) else (
          echo Please disconnect the device and reboot your computer.
          echo Once rebooted, connect the device and run the flashing program again.
          goto :end
      )
    )

    goto :end


:Trim
    Set %2=%1
    exit /b

:checkIfAdminOrElevated
    net session > NUL 2>&1
    if %errorLevel% NEQ 0 (
        echo Access denied. Administrator privileges are required to install the driver.
        echo Please right-click on %~n0%~x0 and select "Run as administrator".
        exit /b 1
    ) else (
        exit /b 0
    )

:checkWindowsArchitecture
    echo Checking Windows architecture...
    echo.
    Set infPathName=.\Driver\x64\arduino-org.inf
    if %PROCESSOR_ARCHITECTURE% == x86 (
        if NOT DEFINED PROCESSOR_ARCHITEW6432 Set infPathName=.\Driver\x86\arduino-org.inf
    )    
    exit /b

:installDriver
    echo Installing the driver...
    pnputil -i -a !infPathName!
    timeout 1 > NUL
    exit /b

:flashDevice
    if "%fwbin%" == "" (
       echo Help us decide what to do. Which device are you trying to flash?
       echo 1 Airspy HF+ DUAL PORT
       echo 2 Airspy HF+ DISCOVERY BB
       echo 3 Airspy HF+ DISCOVERY CD
       set /p vdevice="Enter a number: "
       if !vdevice! == 1 set fwbin=hfplus-firmware-bb.bin
       if !vdevice! == 2 set fwbin=hfplus-firmware-bb.bin
       if !vdevice! == 3 set fwbin=hfplus-firmware-cd.bin
    )
    if "%fwbin%" == "" exit /b
    echo Using binary file %fwbin%
    bossac -u -p!DeviceID!
    timeout 1 > NUL
    bossac.exe -e -b -v -p!DeviceID! -w ./%fwbin%
    timeout 1 > NUL
    echo Rebooting the device in normal mode...
    bossac.exe --reset -p!DeviceID!
    timeout 1 > NUL
    for /f %%i in (Calib) do set Calibration=%%i
    if !Calibration!==0 set Calibration=-152
    echo Restoring the calibration...
    hfplus_ppb.exe !Calibration! > NUL
    echo Done!

:end
del /F Calib 2> NUL
del /F FWVersion 2> NUL
echo Press a key to close.
pause > NUL
